@@ -48,6 +48,16 @@ column_by_type = {c.ch_type: c for c in [
 logger = logging.getLogger(__name__)
 
 
+aliases = [
+    # Begin Geo types
+    ('Point', 'Tuple(Float64, Float64)'),
+    ('Ring', 'Array(Point)'),
+    ('Polygon', 'Array(Ring)'),
+    ('MultiPolygon', 'Array(Polygon)')
+    # End Geo types
+]
+
+
 def get_column_by_spec(spec, column_options, use_numpy=None):
     context = column_options['context']
 
@@ -80,28 +90,45 @@ def get_column_by_spec(spec, column_options, use_numpy=None):
         return create_decimal_column(spec, column_options)
 
     elif spec.startswith('Array'):
-        return create_array_column(spec, create_column_with_options)
+        return create_array_column(
+            spec, create_column_with_options, column_options
+        )
 
     elif spec.startswith('Tuple'):
-        return create_tuple_column(spec, create_column_with_options)
+        return create_tuple_column(
+            spec, create_column_with_options, column_options
+        )
 
     elif spec.startswith('Nested'):
-        return create_nested_column(spec, create_column_with_options)
+        return create_nested_column(
+            spec, create_column_with_options, column_options
+        )
 
     elif spec.startswith('Nullable'):
         return create_nullable_column(spec, create_column_with_options)
 
     elif spec.startswith('LowCardinality'):
-        return create_low_cardinality_column(spec, create_column_with_options)
+        return create_low_cardinality_column(
+            spec, create_column_with_options, column_options
+        )
 
     elif spec.startswith('SimpleAggregateFunction'):
         return create_simple_aggregate_function_column(
-            spec, create_column_with_options)
+            spec, create_column_with_options
+        )
 
     elif spec.startswith('Map'):
-        return create_map_column(spec, create_column_with_options)
+        return create_map_column(
+            spec, create_column_with_options, column_options
+        )
 
     else:
+        for alias, primitive in aliases:
+            if spec.startswith(alias):
+                return create_column_with_options(
+                    primitive + spec[len(alias):]
+                )
+
         try:
             cls = column_by_type[spec]
             return cls(**column_options)
@@ -110,11 +137,11 @@ def get_column_by_spec(spec, column_options, use_numpy=None):
             raise errors.UnknownTypeError('Unknown type {}'.format(spec))
 
 
-def read_column(context, column_spec, n_items, buf):
+def read_column(context, column_spec, n_items, buf, use_numpy=None):
     column_options = {'context': context}
-    column = get_column_by_spec(column_spec, column_options)
-    column.read_state_prefix(buf)
-    return column.read_data(n_items, buf)
+    col = get_column_by_spec(column_spec, column_options, use_numpy=use_numpy)
+    col.read_state_prefix(buf)
+    return col.read_data(n_items, buf)
 
 
 def write_column(context, column_name, column_spec, items, buf,
