@@ -113,6 +113,11 @@ class Connection(object):
     :param ciphers: see :func:`ssl.wrap_socket` docs.
     :param keyfile: see :func:`ssl.wrap_socket` docs.
     :param certfile: see :func:`ssl.wrap_socket` docs.
+    :param server_hostname: Hostname to use in SSL Wrapper construction.
+                            Defaults to `None` which will send the passed
+                            host param during SSL initialization. This param
+                            may be used when connecting over an SSH tunnel
+                            to correctly identify the desired server via SNI.
     :param alt_hosts: list of alternative hosts for connection.
                       Example: alt_hosts=host1:port1,host2:port2.
     :param settings_is_important: ``False`` means unknown settings will be
@@ -135,6 +140,7 @@ class Connection(object):
             # Secure socket parameters.
             verify=True, ssl_version=None, ca_certs=None, ciphers=None,
             keyfile=None, certfile=None,
+            server_hostname=None,
             alt_hosts=None,
             settings_is_important=False,
     ):
@@ -176,6 +182,8 @@ class Connection(object):
 
         self.ssl_options = ssl_options
 
+        self.server_hostname = server_hostname
+
         # Use LZ4 compression by default.
         if compression is True:
             compression = 'lz4'
@@ -209,6 +217,14 @@ class Connection(object):
 
         super(Connection, self).__init__()
 
+    def __repr__(self):
+        dsn = '%s://%s:***@%s:%s/%s' % (
+            'clickhouses' if self.secure_socket else 'clickhouse',
+            self.user, self.host, self.port, self.database
+        ) if self.connected else '(not connected)'
+
+        return '<Connection(dsn=%s, compression=%s)>' % (dsn, self.compression)
+
     def get_description(self):
         return '{}:{}'.format(self.host, self.port)
 
@@ -247,7 +263,8 @@ class Connection(object):
 
                 if self.secure_socket:
                     ssl_context = self._create_ssl_context(ssl_options)
-                    sock = ssl_context.wrap_socket(sock, server_hostname=host)
+                    sock = ssl_context.wrap_socket(
+                        sock, server_hostname=self.server_hostname or host)
 
                 sock.connect(sa)
                 return sock
@@ -482,7 +499,7 @@ class Connection(object):
         packet.type = packet_type = read_varint(self.fin)
 
         if packet_type == ServerPacketTypes.DATA:
-            packet.block = self.receive_data()
+            packet.block = self.receive_data(may_be_use_numpy=True)
 
         elif packet_type == ServerPacketTypes.EXCEPTION:
             packet.exception = self.receive_exception()
@@ -500,8 +517,8 @@ class Connection(object):
             packet.block = self.receive_data()
 
         elif packet_type == ServerPacketTypes.LOG:
-            block = self.receive_data(raw=True)
-            log_block(block)
+            packet.block = self.receive_data(may_be_compressed=False)
+            log_block(packet.block)
 
         elif packet_type == ServerPacketTypes.END_OF_STREAM:
             self.is_query_executing = False
@@ -512,6 +529,15 @@ class Connection(object):
                 packet_type
             )
 
+        elif packet_type == ServerPacketTypes.PART_UUIDS:
+            packet.block = self.receive_data()
+
+        elif packet_type == ServerPacketTypes.READ_TASK_REQUEST:
+            packet.block = self.receive_data()
+
+        elif packet_type == ServerPacketTypes.PROFILE_EVENTS:
+            packet.block = self.receive_data(may_be_compressed=False)
+
         else:
             message = 'Unknown packet {} from server {}'.format(
                 packet_type, self.get_description()
@@ -540,13 +566,15 @@ class Connection(object):
         else:
             return BlockOutputStream(self.fout, self.context)
 
-    def receive_data(self, raw=False):
+    def receive_data(self, may_be_compressed=True, may_be_use_numpy=False):
         revision = self.server_info.revision
 
         if revision >= defines.DBMS_MIN_REVISION_WITH_TEMPORARY_TABLES:
             read_binary_str(self.fin)
 
-        return (self.block_in_raw if raw else self.block_in).read()
+        reader = self.block_in if may_be_compressed else self.block_in_raw
+        use_numpy = False if not may_be_use_numpy else None
+        return reader.read(use_numpy=use_numpy)
 
     def receive_exception(self):
         return read_exception(self.fin)
