@@ -1,5 +1,5 @@
 
-DEFAULT_DATABASE = 'default'
+DEFAULT_DATABASE = ''
 DEFAULT_USER = 'default'
 DEFAULT_PASSWORD = ''
 
@@ -21,6 +21,10 @@ DBMS_MIN_REVISION_WITH_CLIENT_WRITE_INFO = 54420
 DBMS_MIN_REVISION_WITH_SETTINGS_SERIALIZED_AS_STRINGS = 54429
 DBMS_MIN_REVISION_WITH_INTERSERVER_SECRET = 54441
 DBMS_MIN_REVISION_WITH_OPENTELEMETRY = 54442
+DBMS_MIN_PROTOCOL_VERSION_WITH_DISTRIBUTED_DEPTH = 54448
+DBMS_MIN_PROTOCOL_VERSION_WITH_INITIAL_QUERY_START_TIME = 54449
+DBMS_MIN_PROTOCOL_VERSION_WITH_INCREMENTAL_PROFILE_EVENTS = 54451
+DBMS_MIN_REVISION_WITH_PARALLEL_REPLICAS = 54453
 
 # Timeouts
 DBMS_DEFAULT_CONNECT_TIMEOUT_SEC = 10
@@ -36,7 +40,7 @@ CLIENT_NAME = 'python-driver'
 CLIENT_VERSION_MAJOR = 20
 CLIENT_VERSION_MINOR = 10
 CLIENT_VERSION_PATCH = 2
-CLIENT_REVISION = 54442
+CLIENT_REVISION = 54453
 
 BUFFER_SIZE = 1048576
 
