@@ -2,7 +2,7 @@ from datetime import date, datetime
 from enum import Enum
 from uuid import UUID
 
-from .compat import text_type, string_types
+from pytz import timezone
 
 
 escape_chars_map = {
@@ -19,27 +19,36 @@ escape_chars_map = {
 }
 
 
-def escape_param(item):
+def escape_datetime(item, context):
+    server_tz = timezone(context.server_info.timezone)
+
+    if item.tzinfo is not None:
+        item = item.astimezone(server_tz)
+
+    return "'%s'" % item.strftime('%Y-%m-%d %H:%M:%S')
+
+
+def escape_param(item, context):
     if item is None:
         return 'NULL'
 
     elif isinstance(item, datetime):
-        return "'%s'" % item.strftime('%Y-%m-%d %H:%M:%S')
+        return escape_datetime(item, context)
 
     elif isinstance(item, date):
         return "'%s'" % item.strftime('%Y-%m-%d')
 
-    elif isinstance(item, string_types):
+    elif isinstance(item, str):
         return "'%s'" % ''.join(escape_chars_map.get(c, c) for c in item)
 
     elif isinstance(item, list):
-        return "[%s]" % ', '.join(text_type(escape_param(x)) for x in item)
+        return "[%s]" % ', '.join(str(escape_param(x, context)) for x in item)
 
     elif isinstance(item, tuple):
-        return "(%s)" % ', '.join(text_type(escape_param(x)) for x in item)
+        return "(%s)" % ', '.join(str(escape_param(x, context)) for x in item)
 
     elif isinstance(item, Enum):
-        return escape_param(item.value)
+        return escape_param(item.value, context)
 
     elif isinstance(item, UUID):
         return "'%s'" % str(item)
@@ -48,10 +57,10 @@ def escape_param(item):
         return item
 
 
-def escape_params(params):
+def escape_params(params, context):
     escaped = {}
 
     for key, value in params.items():
-        escaped[key] = escape_param(value)
+        escaped[key] = escape_param(value, context)
 
     return escaped
