@@ -1,33 +1,14 @@
-from contextlib import contextmanager
 from datetime import date, datetime
-import os
-from time import tzset
+from unittest.mock import patch
 
-from mock import patch
 from pytz import timezone, utc, UnknownTimeZoneError
 import tzlocal
 
 from tests.testcase import BaseTestCase
-from tests.util import require_server_version
+from tests.util import require_server_version, patch_env_tz
 
 
-class BaseDateTimeTestCase(BaseTestCase):
-    def setUp(self):
-        super(BaseDateTimeTestCase, self).setUp()
-
-        # Bust tzlocal cache.
-        try:
-            tzlocal.unix._cache_tz = None
-        except AttributeError:
-            pass
-
-        try:
-            tzlocal.win32._cache_tz = None
-        except AttributeError:
-            pass
-
-
-class DateTimeTestCase(BaseDateTimeTestCase):
+class DateTimeTestCase(BaseTestCase):
     def test_simple(self):
         with self.create_table('a Date, b DateTime'):
             data = [(date(2012, 10, 25), datetime(2012, 10, 25, 14, 7, 19))]
@@ -82,10 +63,15 @@ class DateTimeTestCase(BaseDateTimeTestCase):
             self.assertEqual(inserted, data)
 
     def test_handle_errors_from_tzlocal(self):
-        with patch('tzlocal.get_localzone') as mocked_get_localzone:
-            mocked_get_localzone.side_effect = UnknownTimeZoneError()
+        with patch('tzlocal.get_localzone') as mocked:
+            mocked.side_effect = UnknownTimeZoneError()
             self.client.execute('SELECT now()')
 
+        if hasattr(tzlocal, 'get_localzone_name'):
+            with patch('tzlocal.get_localzone_name') as mocked:
+                mocked.side_effect = None
+                self.client.execute('SELECT now()')
+
     @require_server_version(20, 1, 2)
     def test_datetime64_frac_trunc(self):
         with self.create_table('a DateTime64'):
@@ -135,22 +121,57 @@ class DateTimeTestCase(BaseDateTimeTestCase):
             inserted = self.emit_cli(query)
             self.assertEqual(inserted, '0\n1\n1500000000\n4294967295\n')
 
+    @require_server_version(21, 4)
+    def test_insert_datetime64_extended_range(self):
+        with self.create_table('a DateTime64(0)'):
+            self.client.execute(
+                'INSERT INTO test (a) VALUES',
+                [(-1420077600, ), (-1420077599, ),
+                 (0, ), (1, ),
+                 (9877248000, )]
+            )
 
-class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
+            query = 'SELECT toInt64(a), a FROM test'
+            inserted = self.emit_cli(query)
+            self.assertEqual(
+                inserted,
+                '-1420077600\t1925-01-01 00:00:00\n'
+                '-1420077599\t1925-01-01 00:00:01\n'
+                '0\t1970-01-01 03:00:00\n'
+                '1\t1970-01-01 03:00:01\n'
+                '9877248000\t2282-12-31 03:00:00\n'
+            )
+            query = 'SELECT a FROM test ORDER BY a'
+            inserted = self.client.execute(query)
+            self.assertEqual(
+                inserted, [
+                    (datetime(1925, 1, 1, 0, 0, 0), ),
+                    (datetime(1925, 1, 1, 0, 0, 1), ),
+                    (datetime(1970, 1, 1, 3, 0, 0), ),
+                    (datetime(1970, 1, 1, 3, 0, 1), ),
+                    (datetime(2282, 12, 31, 3, 0, 0), )
+                ]
+            )
+
+    @require_server_version(21, 4)
+    def test_insert_datetime64_extended_range_pure_ints_out_of_range(self):
+        with self.create_table('a DateTime64(0)'):
+            self.client.execute(
+                'INSERT INTO test (a) VALUES',
+                [(0, ), (1, ), (-2**63, ), (2**63-1, )]
+            )
+
+            query = 'SELECT toInt64(a) FROM test ORDER BY a'
+            inserted = self.emit_cli(query)
+            self.assertEqual(
+                inserted,
+                '-9223372036854775808\n0\n1\n9223372036854775807\n'
+            )
+
+
+class DateTimeTimezonesTestCase(BaseTestCase):
     dt_type = 'DateTime'
 
-    @contextmanager
-    def patch_env_tz(self, tz_name):
-        # Although in many cases, changing the TZ environment variable may
-        # affect the output of functions like localtime() without calling
-        # tzset(), this behavior should not be relied on.
-        # https://docs.python.org/3/library/time.html#time.tzset
-        with patch.dict(os.environ, {'TZ': tz_name}):
-            tzset()
-            yield
-
-        tzset()
-
     # Asia/Kamchatka = UTC+12
     # Asia/Novosibirsk = UTC+7
     # Europe/Moscow = UTC+3
@@ -182,7 +203,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
         offset = timezone(server_tz_name).utcoffset(self.dt).total_seconds()
         timestamp = 1500010800 - int(offset)
 
-        with self.patch_env_tz('Asia/Novosibirsk'):
+        with patch_env_tz('Asia/Novosibirsk'):
             with self.create_table(self.table_columns()):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt, )]
@@ -213,7 +234,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
 
         settings = {'use_client_time_zone': True}
 
-        with self.patch_env_tz('Asia/Novosibirsk'):
+        with patch_env_tz('Asia/Novosibirsk'):
             with self.create_table(self.table_columns()):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt, )],
@@ -249,7 +270,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
         server_tz_name = self.client.execute('SELECT timezone()')[0][0]
         offset = timezone(server_tz_name).utcoffset(self.dt)
 
-        with self.patch_env_tz('Asia/Novosibirsk'):
+        with patch_env_tz('Asia/Novosibirsk'):
             with self.create_table(self.table_columns()):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt_tz, )]
@@ -282,7 +303,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
 
         settings = {'use_client_time_zone': True}
 
-        with self.patch_env_tz('Asia/Novosibirsk'):
+        with patch_env_tz('Asia/Novosibirsk'):
             with self.create_table(self.table_columns()):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt_tz, )],
@@ -319,7 +340,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
         # into column with timezone Asia/Novosibirsk
         # using server's timezone (Europe/Moscow)
 
-        with self.patch_env_tz('Europe/Moscow'):
+        with patch_env_tz('Europe/Moscow'):
             with self.create_table(self.table_columns(with_tz=True)):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt, )]
@@ -355,7 +376,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
 
         settings = {'use_client_time_zone': True}
 
-        with self.patch_env_tz('Europe/Moscow'):
+        with patch_env_tz('Europe/Moscow'):
             with self.create_table(self.table_columns(with_tz=True)):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt, )],
@@ -390,7 +411,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
         # into column with timezone Asia/Novosibirsk
         # using server's timezone (Europe/Moscow)
 
-        with self.patch_env_tz('Europe/Moscow'):
+        with patch_env_tz('Europe/Moscow'):
             with self.create_table(self.table_columns(with_tz=True)):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt_tz, )]
@@ -430,7 +451,7 @@ class DateTimeTimezonesTestCase(BaseDateTimeTestCase):
 
         settings = {'use_client_time_zone': True}
 
-        with self.patch_env_tz('Europe/Moscow'):
+        with patch_env_tz('Europe/Moscow'):
             with self.create_table(self.table_columns(with_tz=True)):
                 self.client.execute(
                     'INSERT INTO test (a) VALUES', [(self.dt_tz, )],
