@@ -4,6 +4,7 @@ from unittest.mock import patch
 from clickhouse_driver.errors import ServerException
 from tests.testcase import BaseTestCase, file_config
 from tests.util import capture_logging
+from clickhouse_driver.util.helpers import chunks
 
 
 class BlocksTestCase(BaseTestCase):
@@ -149,10 +150,27 @@ class IteratorTestCase(BaseTestCase):
             'SELECT number FROM system.numbers LIMIT 10'
         )
         self.assertIsInstance(result, types.GeneratorType)
-
         self.assertEqual(list(result), list(zip(range(10))))
         self.assertEqual(list(result), [])
 
+    def test_select_with_chunk_one_iter(self):
+        result = self.client.execute_iter(
+            'SELECT number FROM system.numbers LIMIT 10',
+            chunk_size=1
+        )
+        self.assertIsInstance(result, types.GeneratorType)
+        self.assertEqual(list(result), list(zip(range(10))))
+        self.assertEqual(list(result), [])
+
+    def test_select_with_chunk_some_iter(self):
+        result = self.client.execute_iter(
+            'SELECT number FROM system.numbers LIMIT 10',
+            chunk_size=3
+        )
+        self.assertIsInstance(result, types.GeneratorType)
+        self.assertEqual(list(result), list(chunks(zip(range(10)), 3)))
+        self.assertEqual(list(result), [])
+
     def test_select_with_iter_with_column_types(self):
         result = self.client.execute_iter(
             'SELECT CAST(number AS UInt32) as number '
