@@ -19,7 +19,7 @@ class PacketsTestCase(BaseTestCase):
         self.assertEqual(ClientPacketTypes.to_str(42), 'Unknown packet')
 
         self.assertEqual(ServerPacketTypes.to_str(4), 'Pong')
-        self.assertEqual(ServerPacketTypes.to_str(12), 'Unknown packet')
+        self.assertEqual(ServerPacketTypes.to_str(15), 'Unknown packet')
         self.assertEqual(ServerPacketTypes.to_str(42), 'Unknown packet')
 
 
@@ -240,6 +240,30 @@ class ConnectTestCase(BaseTestCase):
         list(self.client.execute_iter('SELECT 1'))
         list(self.client.execute_iter('SELECT 1'))
 
+    def test_round_robin(self):
+        kwargs = {
+            'round_robin': True,
+            'alt_hosts': '{}:{}'.format(self.host, self.port)
+        }
+        with self.created_client(**kwargs) as client:
+            self.assertFalse(client.connection.connected)
+            self.assertFalse(list(client.connections)[0].connected)
+
+            client.execute('SELECT 1')
+
+            self.assertTrue(client.connection.connected)
+            self.assertFalse(list(client.connections)[0].connected)
+
+            client.execute('SELECT 1')
+
+            self.assertTrue(client.connection.connected)
+            self.assertTrue(list(client.connections)[0].connected)
+
+            client.disconnect()
+
+            self.assertFalse(client.connection.connected)
+            self.assertFalse(list(client.connections)[0].connected)
+
 
 class FakeBufferedReader(BufferedReader):
     def __init__(self, inputs, bufsize=128):
